;; WIN.LSP
;;
;; By Daniel Hargreaves
;; October 10, 1994
;; Version 1.0
;;
;; This routine draws windows with multiple mullions 
;; in plan view between any two lines at any angle, 
;; witout the use of a block. It works best if 
;; the lines are parallel, but is not required.
;; The routine will place the mullions and glass
;; on the current layer, and it will place the wall
;; jambs on the same layer as the walls.
;;
;; I have included two functions. One of them is for
;; Command line prompts C:WIN and the other is a Dialog 
;; box C:DDWIN for user input of the Window & Mullion 
;; sizes.
;;
;; This LISP routine is being distributed as Postcard Ware
;; which means if you like and find it useful it send me a 
;; postcard and if you don't then delete it. To register 
;; Simply send a postcard with the name of your hometown 
;; written on it to:
;;
;; Daniel Hargreaves
;; 13583 Red Fern Lane
;; Dallas, TX 75240
;;
;; Rev 1.1 Dialog Bug Fix - November 5, 1994
;; Rev 1.2 Unlock Wall Layer - Novenber 8, 1994

;; Save System Varibles
(defun SSV (savelist)
  (mapcar
    '(lambda (sysvar)
      (list sysvar (getvar sysvar))
    )
  savelist)
)
(setq lista '("blipmode"  "cecolor" "clayer" "cmdecho" 
"orthomode" "osmode" "snapmode"))

;; Reset System Varibles
(defun RSV (savelist)
  (mapcar
    '(lambda (sysvar)
      (setvar (car sysvar)(cadr sysvar))
      (car sysvar)
    )
  savelist)
)

;; Error handler
(defun err ()
  (command ".undo" "mark")
  (setq olderr *error*)
  (defun *error* (errstr)
    (print errstr)(princ)
    (setq lista (RSV lista))
    (setq *error* olderr)
    (princ)
  )
  (setq lista (SSV lista))
  (setvar "blipmode" 0)
  (setvar "cmdecho" 0)
  (setvar "orthomode" 0)
  (setvar "snapmode" 0)
)

(defun reset ()
  (setq lista (RSV lista))
  (setq *error* olderr)
)

;; GetMidpoint routine
(defun gm ( / lst lpt1 lpt2 mx my midp )
  (setq lst (entget (entlast)))
  (setq lpt1 (cdr (assoc 10 lst)))
  (setq lpt2 (cdr (assoc 11 lst)))
  (setq lpt1 (car (list lpt1 lpt2)))
  (setq lpt2 (cadr (list lpt1 lpt2)))
  (setq mx (* (+ (nth 0 lpt1) (nth 0 lpt2)) 0.5))
  (setq my (* (+ (nth 1 lpt1) (nth 1 lpt2)) 0.5))
  (setq midp (list mx my))
)

;; Main Defun
(defun win ( / cl ce pb ap line1 pt#1 pt#2 pt#3 pt#4 pm#1 pm#2 pm#3 pm#4 
               ang1 ang2 wan1 wan2 jm#1 jm#2 mp#1 mp#2 i# mdst fstm nxtm
               ml#1 gp#1 gp#2 cla tla l70)

  (setq cl (getvar "clayer")
        ce (getvar "cecolor")
  )

;; Save variables for next time
  (if (not wsz) (setq wsz *wsz) (setq *wsz wsz))
  (if (not mlx) (setq mlx *mlx) (setq *mlx mlx))
  (if (not mly) (setq mly *mly) (setq *mly mly))
  (if (not lit) (setq lit *lit) (setq *lit lit))

;; Find the wall points

  (setq line1 (entsel "\nSelect first wall line to be opened: ")
        pt#1 (nth 1 line1))
  (setvar "osmode" 128)
  (setq pt#2 (getpoint pt#1 "\nSelect opposite face to be opened: ")
        ang1 (angle pt#1 pt#2)
        ang2 (angle pt#2 pt#1))
  (setvar "osmode" 0)
  (cond
    ((= lmr "Left") (setq wan1 (+ ang1 1.5708) 
                          wan2 (+ ang2 1.5708)))
    ((= lmr "Right") (setq wan1 (+ ang1 4.71239) 
                           wan2 (+ ang2 4.71239)))
    ((= lmr "Middle") (setq wang (+ ang1 1.5708)
                            pt#1 (polar pt#1 wang (* wsz 0.5))
                            pt#2 (polar pt#2 wang (* wsz 0.5))
                            wan1 (+ ang1 4.71239)
                            wan2 (+ ang2 4.71239)))
  )
  (setq pt#3 (polar pt#1 wan1 wsz)
        pt#4 (polar pt#2 wan1 wsz)
        mp#1 (polar pt#1 wan1 (* wsz 0.5))
        mp#2 (polar pt#2 wan1 (* wsz 0.5))
  )

;; Cut the opening
  (setq l1_list (entget (car line1)))
  (setvar "clayer" (cdr (assoc 8 l1_list)))
  (cond
    ((setq c_list (assoc 62 l1_list)) 
      (setq color (cdr cr_list))
      (command ".color" color)
    )
  )

;; Check for locked layer
  (setq cla (getvar "clayer")
        tla (tblsearch "layer" cla)
        l70 (cdr (assoc 70 tla))
  )
  (if (= l70 68)
      (command "layer" "unlock" cla "")
  )

  (command ".break" pt#1 pt#3 ".break" pt#2 pt#4)
  (setvar "osmode" 512)
  (command ".line" pt#1 pt#2 "")
  (setq jm#1 (gm)) 
  (command ".line" pt#3 pt#4 "")
  (setq jm#2 (gm))

  (if (= l70 68)
      (command "layer" "lock" cla "")
  )
  (setvar "clayer" cl)
  (setvar "cecolor" ce)
  (setvar "osmode" 0)

;; Build the mullions 
  (setq pm#1 (polar jm#1 ang1 (* mly 0.5))
        pm#2 (polar pm#1 wan1 mlx)
        pm#3 (polar pm#2 ang2 mly)
        pm#4 (polar pm#3 wan2 mlx)
        jm#1 (polar pm#2 ang2 (* mly 0.5))
  )
  (command ".pline" pm#1 pm#2 pm#3 pm#4 "close")
  (setq ml#1 (entlast))

;; Build the rest of the mullions & glass
  (setq i# 0
        mdst (/ (- wsz mlx) lit)
        fstm (polar jm#1 wan2 (* mlx 0.5))
        nxtm (polar fstm wan1 mdst)
        gp#1 (polar fstm wan1 (* mlx 0.5))
        gp#2 (polar nxtm wan2 (* mlx 0.5))
  )

  (While (< i# lit)
    (command "copy" "single" ml#1 fstm nxtm )
    (setq ml#1 (entlast))
    (command "line" gp#1 gp#2 "")
    (setq fstm nxtm
          nxtm (polar fstm wan1 mdst)
          gp#1 (polar fstm wan1 (* mlx 0.5))
          gp#2 (polar nxtm wan2 (* mlx 0.5))

    )
    (Setq i# (1+ i#))
  )

)

;; --Command line functions--

(defun c:win ( / aslu lmr wsz lit mlx mly )

  (err)
  (initget "Left Middle Right")
  (setq aslu (getvar "lunits")
        lmr (getkword "\nInsertion point Left, Middle or <Right>: "))
  (if (= lmr nil)
    (setq lmr "Right")
  )

;; Window size
  (if (not *wsz) (setq *wsz 48.0))
  (princ "\nEnter size of window opening <")
  (princ (rtos *wsz aslu))
  (setq wsz (getdist ">: "))

;; Number of Windows
  (if (not *lit) (setq *lit 1))
  (princ "\nEnter number of window lites <")
  (princ *lit)
  (initget 6)
  (setq lit (getint ">: "))

;; Mullion size
  (if (not *mlx) (setq *mlx 2.0))
  (if (not *mly) (setq *mly 4.0))
  (princ "\nEnter length of mullion <")
  (princ (rtos *mlx aslu))
  (setq mlx (getdist ">: "))
  (princ "\nEnter size of window opening <")
  (princ (rtos *mly aslu))
  (setq mly (getdist ">: "))
  (win)
  (reset)
  (princ)
)

;; --Dialog Box functions--

(defun c:ddwin ( / wsz mlx mly lit lmr stwn )

  (err)
  (if (not *wsz) (setq *wsz 48.0))
  (if (not *mlx) (setq *mlx 2.0))
  (if (not *mly) (setq *mly 4.0))
  (if (not *lit) (setq *lit 1))
  (if (not *lmr) (setq *lmr 1))

  (SETQ DCL_ID (LOAD_DIALOG "win.dcl"))
  (if (not (NEW_DIALOG "win" DCL_ID)) (exit))
  (set_tile "WSZ" (rtos *wsz))
  (set_tile "MLX" (rtos *mlx))
  (set_tile "MLY" (rtos *mly))
  (set_tile "LIT" (itoa *lit))
  (cond
    ((= *lmr 1) (set_tile "L" "1") (setq lmr 1))
    ((= *lmr 2) (set_tile "R" "1") (setq lmr 2))
    ((= *lmr 3) (set_tile "M" "1") (setq lmr 3))
  )
  (action_tile "WSZ" "(setq wsz (distof $value))")
  (action_tile "MLX" "(setq mlx (distof $value))")
  (action_tile "MLY" "(setq mly (distof $value))")
  (action_tile "L" "(setq lmr 1)")
  (action_tile "R" "(setq lmr 2)")
  (action_tile "M" "(setq lmr 3)")
  (action_tile "LIT" "(setq lit (atoi $value))")
  (action_tile "accept" "(setq stwn 1) (done_dialog 1)")
  (action_tile "cancel" "(done_dialog)")
  (START_DIALOG)
  (unload_dialog dcl_id)
  (cond
    ((= lmr 1) (setq lmr "Left" *lmr 1))
    ((= lmr 2) (setq lmr "Right" *lmr 2))
    ((= lmr 3) (setq lmr "Middle" *lmr 3))
  )
  (if (= stwn 1) (win))
  (reset)
  (princ)
)

(princ "\nC:WIN Loaded - Ver 1.2 By Daniel Hargreaves.")
(princ "\nStart Command with WIN or Dialog Box with DDWIN.")
(princ)
